// Codage/decodage de textes morse (et rot13)
// version 0.5 - 08-05-2000
// ric Lvnez : <eric.levenez@wanadoo.fr>

#import "Morse.h"

@implementation Morse

// Le buffer de saisie

static char buf[BUFSIZ];

// Codage des lettres et des chiffres */
// Les codes morses sont codes en hexadecimal avec 0="." et 1="-".
// Par exemple "-.-" vaut en decimal 5 et a une longueur de 3

static const struct {
	char	l;	// longueur du code morse, ou 0 si 'c' est non traduit
	char	c;	// codage hexa du morse, ou caractere a afficher si 'l' vaut 0
} morse[] = {

	// Codes action ASCII

	{ 0, '\000' },	{ 0, '\001' },	{ 0, '\002' },	{ 0, '\003' },
	{ 0, '\004' },	{ 0, '\005' },	{ 0, '\006' },	{ 0, '\007' },
	{ 0, '\010' },	{ 0, '\011' },	{ 6, 20 },	{ 0, '\013' },
	{ 0, '\014' },	{ 0, '\015' },	{ 0, '\016' },	{ 0, '\017' },
	{ 0, '\020' },	{ 0, '\021' },	{ 0, '\022' },	{ 0, '\023' },
	{ 0, '\024' },	{ 0, '\025' },	{ 0, '\026' },	{ 0, '\027' },
	{ 0, '\030' },	{ 0, '\031' },	{ 0, '\032' },	{ 0, '\033' },
	{ 0, '\034' },	{ 0, '\035' },	{ 0, '\036' },	{ 0, '\037' },

	// Caracteres speciaux space ! " # $ % & ' ( ) * + , - . /

	{ 0, ' ' },	{ 0, '!' },	{ 6, 18 },	{ 0, '#' },
	{ 0, '$' },	{ 0, '%' },	{ 0, '&' },	{ 6, 30 },
	{ 5, 22 },	{ 5, 29 },	{ 0, '*' },	{ 4, 5 },
	{ 6, 51 },	{ 6, 33 },	{ 6, 21 },	{ 5, 18 },

	// Chiffres 0 1 2 3 4 5 6 7 8 9

	{ 5, 31 },	{ 5, 15 },	{ 5, 7 },	{ 5, 3 },
	{ 5, 1 },	{ 5, 0 },	{ 5, 16 },	{ 5, 24 },
	{ 5, 28 },	{ 5, 30 },

	// Caracteres speciaux : ; < = > ? @

	{ 6, 56 },	{ 6, 42 },	{ 0, '<' },	{ 5, 17 },
	{ 0, '>' },	{ 6, 12 },	{ 0, '@' },

	// Lettres majuscules A B C ... X Y Z

	{ 2, 1 },	{ 4, 8 },	{ 4, 10 },	{ 3, 4},
	{ 1, 0 },	{ 4, 2 },	{ 3, 6 },	{ 4, 0 },
	{ 2, 0 },	{ 4, 7 },	{ 3, 5 },	{ 4, 4 },
	{ 2, 3 },	{ 2, 2},	{ 3, 7 },	{ 4, 6 },
	{ 4, 13 },	{ 3, 2 },	{ 3, 0 },	{ 1, 1},
	{ 3, 1 },	{ 4, 1 },	{ 3, 3 },	{ 4, 9 },
	{ 4, 11 },	{ 4, 12 },

	// Caracteres speciaux [ \ ] ^ _ `

	{ 0, '[' },	{ 5, 18 },	{ 0, ']' },	{ 0, '^' },
	{ 6, 13 },	{ 6, 30 },

	// Lettres minuscules a b c ... x y z

	{ 2, 1 },	{ 4, 8 },	{ 4, 10 },	{ 3, 4 },
	{ 1, 0 },	{ 4, 2 },	{ 3, 6 },	{ 4, 0 },
	{ 2, 0 },	{ 4, 7 },	{ 3, 5 },	{ 4, 4 },
	{ 2, 3 },	{ 2, 2 },	{ 3, 7 },	{ 4, 6 },
	{ 4, 13 },	{ 3, 2 },	{ 3, 0 },	{ 1, 1 },
	{ 3, 1 },	{ 4, 1 },	{ 3, 3 },	{ 4, 9 },
	{ 4, 11 },	{ 4, 12 },

	// Caracteres speciaux { | } ~ del

	{ 0, '{' },	{ 0, '|' },	{ 0, '}' },	{ 0, '~' },
	{ 0, '\177' },

	// Caracteres speciaux (avec l'encodage NeXTSTEP des accents)

	{ 0, '\200' },	{ 2, 1 },	{ 2, 1 },	{ 2, 1 },
	{ 2, 1 },	{ 2, 1 },	{ 2, 1 },	{ 4, 10 },
	{ 1, 0 },	{ 1, 0 },	{ 1, 0 },	{ 1, 0 },
	{ 2, 0 },	{ 2, 0 },	{ 2, 0 },	{ 2, 0 },
	{ 3, 4 },	{ 2, 2 },	{ 3, 7 },	{ 3, 7 },
	{ 3, 7 },	{ 3, 7 },	{ 3, 7 },	{ 3, 1 },
	{ 3, 1 },	{ 3, 1 },	{ 3, 1 },	{ 4, 11 },
	{ 0, '\234' },	{ 0, '\235' },	{ 4, 9 },	{ 5, 18 },
	{ 4, 10 },	{ 0, '\241' },	{ 4, 10 },	{ 0, '\243' },
	{ 0, '\244' },	{ 4, 11 },	{ 4, 2 },	{ 0, '\247' },
	{ 0, '\250' },	{ 6, 30 },	{ 0, '\252' },	{ 0, '\253' },
	{ 0, '\254' },	{ 0, '\255' },	{ 0, '\256' },	{ 0, '\257' },
	{ 3, 2 },	{ 6, 33 },	{ 0, '\262' },	{ 0, '\263' },
	{ 6, 21 },	{ 0, '\265' },	{ 0, '\266' },	{ 6, 21 },
	{ 6, 51 },	{ 0, '\271' },	{ 0, '\272' },	{ 6, 12 },
	{ 5, 15 },	{ 6, 30 },	{ 6, 30 },	{ 0, '\277' },
	{ 0, '\300' },	{ 0, '\301' },	{ 0, '\302' },	{ 0, '\303' },
	{ 0, '\304' },	{ 0, '\305' },	{ 0, '\306' },	{ 0, '\307' },
	{ 0, '\310' },	{ 5, 7 },	{ 0, '\312' },	{ 0, '\313' },
	{ 5, 3 },	{ 0, '\315' },	{ 0, '\316' },	{ 0, '\317' },
	{ 6, 33 },	{ 0, '\321' },	{ 0, '\322' },	{ 0, '\323' },
	{ 0, '\324' },	{ 2, 1 },	{ 2, 1 },	{ 2, 1 },
	{ 2, 1 },	{ 2, 1 },	{ 2, 1 },	{ 4, 10 },
	{ 1, 0 },	{ 1, 0 },	{ 1, 0 },	{ 1, 0 },
	{ 2, 0 },	{ 2, 1 },	{ 2, 0 },	{ 0, '\343' },
	{ 2, 0 },	{ 2, 0 },	{ 3, 7 },	{ 2, 2 },
	{ 4, 4 },	{ 3, 7 },	{ 3, 7 },	{ 0, '\353' },
	{ 3, 7 },	{ 3, 7 },	{ 3, 7 },	{ 3, 7 },
	{ 3, 7 },	{ 2, 1 },	{ 3, 1 },	{ 3, 1 },
	{ 3, 1 },	{ 5, 15 },	{ 3, 1 },	{ 4, 11 },
	{ 0, '\370' },	{ 3, 7 },	{ 3, 7 },	{ 4, 8 },
	{ 0, '\374' },	{ 4, 11 },	{ 0, '\376' },	{ 0, '\377' }
};

// Table de transcodage pour le rot13 (a-m et n-z invers)

static const unsigned char tr[256] = {
          0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
         16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,
         32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,
         48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,
         64,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  65,  66,
         67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  91,  92,  93,  94,  95,
         96, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122,  97,  98,
         99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 123, 124, 125, 126, 127,
        128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143,
        144, 145, 146, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157, 158, 159,
        160, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170, 171, 172, 173, 174, 175,
        176, 177, 178, 179, 180, 181, 182, 183, 184, 185, 186, 187, 188, 189, 190, 191,
        192, 193, 194, 195, 196, 197, 198, 199, 200, 201, 202, 203, 204, 205, 206, 207,
        208, 209, 210, 211, 212, 213, 214, 215, 216, 217, 218, 219, 220, 221, 222, 223,
        224, 225, 226, 227, 228, 229, 230, 231, 232, 233, 234, 235, 236, 237, 238, 239,
        240, 241, 242, 243, 244, 245, 246, 247, 248, 249, 250, 251, 252, 253, 254, 255
};

// Table de transcodage pour le rot13 (a-m et n-z invers) en tenant compte des accents

static const unsigned char tr_accents[256] = {
          0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
         16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,
         32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,
         48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,
         64,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  65,  66,
         67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  91,  92,  93,  94,  95,
         96, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122,  97,  98,
         99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 123, 124, 125, 126, 127,
        128,  78,  78,  78,  78,  78,  78,  80,  82,  82,  82,  82,  86,  86,  86,  86, 
         81,  65,  66,  66,  66,  66,  66,  72,  72,  72,  72,  76, 156, 157, 158, 159, 
        160, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170, 171, 172, 173, 174, 175, 
        176, 177, 178, 179, 180, 181, 182, 183, 184, 185, 186, 187, 188, 189, 190, 191, 
        192, 193, 194, 195, 196, 197, 198, 199, 200, 201, 202, 203, 204, 205, 206, 207, 
        208, 209, 210, 211, 212, 110, 110, 110, 110, 110, 110, 112, 114, 114, 114, 114, 
        118, 225, 118, 227, 118, 118,  98,  97,  89, 233, 234, 235,  98,  98,  98,  98, 
         98, 241, 104, 104, 104, 245, 104, 108, 248, 249, 250, 251, 252, 108, 254, 255
};

// Ordre d'analyse du tableau de decodage (on a les minuscules avant les majuscules)

static const unsigned char ordre[256] = {
	39,  97,  98,  99,  100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111,
	112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 64,  65,  66,  67,  68,
	69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,
	85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,  123, 124, 125, 126, 127,
	32,  33,  34,  35,  36,  37,  38,  96,  40,  41,  42,  43,  44,  45,  46,  47,
	48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,
	0,   1,   2,   3,   4,   5,   6,   7,   8,   9,   10,  11,  12,  13,  14,  15,
	16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,
	128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143,
	144, 145, 146, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157, 158, 159,
	160, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170, 171, 172, 173, 174, 175,
	176, 177, 178, 179, 180, 181, 182, 183, 184, 185, 186, 187, 188, 189, 190, 191,
	192, 193, 194, 195, 196, 197, 198, 199, 200, 201, 202, 203, 204, 205, 206, 207,
	208, 209, 210, 211, 212, 213, 214, 215, 216, 217, 218, 219, 220, 221, 222, 223,
	224, 225, 226, 227, 228, 229, 230, 231, 232, 233, 234, 235, 236, 237, 238, 239,
	240, 241, 242, 243, 244, 245, 246, 247, 248, 249, 250, 251, 252, 253, 254, 255
};

/*
 * Lorsque l'application demarre
 */

- appDidInit:sender
{

	// Prevenir des services Morse le systeme

	[[NXApp appListener] setServicesDelegate: self];

	// Definir les bornes de l'ascenseur

	[SlideValue setMinValue:(double)LARGEUR_MIN];
	[SlideValue setMaxValue:(double)LARGEUR_MAX];

	// Lire les valeurs par defaut

	largeur = atoi(NXGetDefaultValue([NXApp appName], "Largeur"));
	if (largeur < LARGEUR_MIN || largeur > LARGEUR_MAX)
		largeur = LARGEUR;
	strict = strcmp(NXGetDefaultValue([NXApp appName], "Strict"), "Yes") ? NO : YES;

	// Modifier l'interface pour refleter ces valeurs

	[StrictValue setState:strict];
	[SlideValue setIntValue:largeur];
	if (largeur == LARGEUR_MAX) {
		largeur = LARGEUR_SANS;

		// Changer le nom de la position maximale

		strcpy(buf, NXLocalizedString("infini", "infini", Largeur infinie des lignes));
	} else
		sprintf(buf, "%d", largeur);
	[SlideTexte setStringValue:buf];

	return self;
}

/*
 * Initialisation des valeurs par defaut
 */

+ initialize
{
#define str(x)	#x

	// Definition des valeurs par defaut

	const NXDefaultsVector MorseDefaults = {
		{ "Largeur",	str(LARGEUR) },
		{ "Strict",	"No" },
		{ NULL,		NULL }
	};

	// Enregistrer ces valeurs

	NXRegisterDefaults("Morse", MorseDefaults);
	return self;
}

/*
 * Gestion des boutons d'encodage et de decodage
 */

- Boutons:sender
{
	int	offset = 0, nb;
	int	type;

	code = CODE_DEBUT;
	caractere = 0;
	type = [[sender selectedCell] tag];
	rot13 = (type < ENCODE_MORSE_ROT13) ? NO : YES;

	// On selectionne tout le texte de sortie pour l'ecraser

	[Sortie selectAll:self];

	// Boucler sur toutes les donnees de la zone de saisie

	for (;; offset += nb) {

		// Lire un bloc

		nb = [Saisie getSubstring:buf start:offset length:BUFSIZ-1];
		if (nb <= 0)
			break;
		buf[nb] = 0;

		// Suivant l'action demandee, decoder ou encoder le buffer

		switch (type) {
		case ENCODE_MORSE :
		case ENCODE_MORSE_ROT13 :
			[self Encode: buf];
			break;

		case DECODE_MORSE :
		case DECODE_MORSE_ROT13 :
			[self Decode: buf];
			break;

		case ENCODE_ROT13 :
			[self Rot: buf];
		}
	}
	return self;
}

/*
 * Gestion de l'ascenseur
 */

- Slide:sender
{
	char	buf[80];

	// Lire la position de l'ascenseur

	largeur = [[sender cell] intValue];

	// Sauvegarder la position

	sprintf(buf, "%d", largeur);
	NXWriteDefault([NXApp appName], "Largeur", buf);

	// La derniere position de l'ascenseur est la position infinie

	if (largeur == LARGEUR_MAX) {
		largeur = LARGEUR_SANS;

		// Changer le nom de la position maximale

		strcpy(buf, NXLocalizedString("infini", "infini", Largeur infinie des lignes));
	}

	// Mettre a jour le champ texte

	[SlideTexte setStringValue:buf];
	return self;
}

/*
 * Gestion du bouton strict
 */

- Strict:sender
{
	// Lire la valeur du bouton

	strict = [sender state];

	// La memoriser

	NXWriteDefault([NXApp appName], "Strict", strict ? "Yes" : "No");
	return self;
}

/*
 * Decodage d'un code morse en chaine de caractere
 */

- Decode:(const char *)p
{
	int	i, j;			// Variables de boucle
	char	l;			// Longueur du pattern
	char	c;			// Code hexa du pattern
	const char *s;			// Pointeur sur les caracteres a decoder
	char	d;			// Le caractere a affiche

	// Boucler sur tous les mots

	for (s = p; *p; p = s) {

		// Coder en hexa le mot courant

		for (l = c = 0; l < 6; l++, s++) {
			switch (*s) {
			case '.' :		// Les points du codage NeXTSTEP
			case '\264' :
			case '\267' :
				c <<= 1;
				continue;

			case '-' :		// Les tirets du codage NeXTSTEP
			case '\261' :
			case '\320' :
				c = (c << 1) | 1;
				continue;
			}
			break;
		}

		// Si l'on n'a rien trouve

		if (!l) {
			d = *p;

			// Si l'on est strict

			if (strict) {

				// On n'accepte que les espaces

				if (strchr(" \t\r\n", d))
					[Sortie replaceSel:" "];
			} else {

				// Remplacer les espaces divers

				if (strchr("\r\n", d))
					d = ' ';

				// Si l'on avait un separateur a mettre

				if ((code == CODE_MORSE) && (d != ' '))
					[Sortie replaceSel:" "];

				// Afficher le caractere tel quel

				[Sortie replaceSel: p length:1];

				// Suivant le caractere affiche

				if (strchr(".?!", d))
					code = CODE_DEBUT;
				else {
					if (!strchr(" \t\r\n", *p))
						code = CODE_NON_MORSE;
				}
			}

			// Passer au caractere suivant

			s++;
			continue;
		}

		// Rechercher le code morse en hexa

		for (i = 0; i < 256; i++) {

			// Si l'on n'est pas au debut d'une phrase, changer l'ordre de
			// recherche dans le tableau pour commencer par les minuscules

			j = (code == CODE_DEBUT) ? i : ordre[i];

			// Sauter les caracteres non morsables

			if (!morse[j].l)
				continue;

			// Si le caractere morse hexa correspond exactement

			if (l == morse[j].l && c == morse[j].c) {

				// Ajouter un blanc comme separateur

				if (code == CODE_MORSE)
					[Sortie replaceSel:" "];

				// Si l'on veut du rot13, faire la conversion

				d = rot13 ? tr[j] : j;

				// Afficher le caractere traduit

				[Sortie replaceSel:&d length:1];

				// Si le caractere suivant est un espace, le sauter

				if (strchr(" \t\r\n", *s))
					s++;

				// Suivant le caractere affiche

				code = strchr(".?!\n", d) ? CODE_DEBUT : CODE_NON_MORSE;
				break;
			}
		}

		// Sequence inconnue

		if (i == 256 && !strict) {

			// Si l'on avait un separateur a mettre

			if (code == CODE_MORSE)
				[Sortie replaceSel:" "];

			// Afficher le mot (code morse inconnu)

			[Sortie replaceSel:p length:(int)(s-p)];

			// Prevoir un separateur

			code = CODE_MORSE;
		}
	}
	return self;
}

/*
 * Codage en morse d'une chaine de caracteres
 */

- Encode:(const char *)p
{
	unsigned char c, d;		// Le caractere sur un nombre de 0 a 255
	const char *separateur = NULL;	// Si le caractere est un separateur
	int	lg;			// Longueur du code morse

	// Boucler sur toutes les lettres

	while ((c = (unsigned char)*p++)) {

		// Si c'est un retour a la ligne

		if ((c == '\n') && (code != CODE_DEBUT))
			c = ' ';

		// Si l'on veut faire du rot13

		if (rot13)
			c = tr_accents[c];
		d = morse[c].c;

		// Si c'est un code "morsable"

		if ((lg = morse[c].l)) {

			// Si l'on doit aller a la ligne

			if ((largeur != LARGEUR_SANS) && ((caractere+lg+1) > largeur)) {
				[Sortie replaceSel:"\n"];
				caractere = lg;
			} else {

				// Ajouter un blanc comme separateur

				if (code != CODE_DEBUT) {
					[Sortie replaceSel:" "];
					caractere++;
				}
				caractere += lg;
			}

			// Le decoder

			while (--lg >= 0)
				[Sortie replaceSel: ((d>>lg)&1) ? "-" : "."];

			// Prevoir un separateur

			code = CODE_MORSE;
		} else {
			separateur = strchr(" \t\r\n", d);

			// Si l'on doit aller a la ligne

			if ((largeur != LARGEUR_SANS) && ((caractere+2) > largeur)) {
				[Sortie replaceSel:"\n"];
				caractere = 1;
			} else {

				// Si l'on avait un separateur a mettre

				if (code == CODE_MORSE) {
					[Sortie replaceSel:" "];
					caractere++;
				}
				caractere++;
			}

			// Si l'on veut faire un codage strict

			if (strict) {

				// Afficher un blanc a la place du caractere

				[Sortie replaceSel:" "];
				code = CODE_DEBUT;
			} else {

				// Si c'est un retour a la ligne

				if (strchr("\r\n", d))
					caractere = 0;

				// Afficher le caractere tel quel

				[Sortie replaceSel:&d length:1];

				// Suivant le caractere affiche

				code = separateur ? CODE_DEBUT : CODE_NON_MORSE;
			}
		}
	}
	return self;
}

/*
 * Codage en morse d'une chaine de caracteres
 */

- Rot:(char *)p
{
	unsigned char *s = (unsigned char *)p;
	int	lg = 0;

	// Inverser toutes les lettres

	while (*s) {
		*s = tr[*s];
		s++;
		lg++;
	}

	// Afficher le resultat

	[Sortie replaceSel:p length:lg];
	return self;
}

/*
 * Decodage d'un code morse en chaine de caractere
 */

- MorseService:(id)pasteboard userData:(const char *)type error:(const char **)msg
{
	char	*p;
	int	lg;
	int	offset = 0, nb;
	int	ntype;

	code = CODE_DEBUT;
	caractere = 0;
	largeur = LARGEUR;

	// Recuperer le type de service demande

	ntype = atoi(type);
	if (ntype > ENCODE_ROT13) {
	        *msg = NXLocalizedString("MauvaisService", "Mauvais paramtrage des services", Mauvaise valeur userData pour les services);
		return self;
	}
	rot13 = (ntype < ENCODE_MORSE_ROT13) ? NO : YES;

	// Si le presse-papier a des donnees que l'on sait traiter

	[pasteboard types];
	if ([pasteboard readType:NXAsciiPboardType data:&p length:&lg]) {

		// Remplacer le texte de la fenetre de saisie

		[Saisie selectAll: self];
		[Saisie replaceSel:p length:lg];
		[Sortie selectAll:self];

		// Decoder la saisie

		for (;; offset += nb) {
			nb = [Saisie getSubstring:buf start:offset length:BUFSIZ-1];
			if (nb <= 0)
				break;
			buf[nb] = 0;
			switch (ntype) {
			case ENCODE_MORSE :
			case ENCODE_MORSE_ROT13 :
				[self Encode:buf];
				break;

			case DECODE_MORSE :
			case DECODE_MORSE_ROT13 :
				[self Decode:buf];
				break;

			case ENCODE_ROT13 :
				[self Rot:buf];
			}
		}

		// Activer la fenetre resultat

		[window makeKeyAndOrderFront:self];
	} else {

		// Si l'on n'a pas de texte ASCII dans le pastboard

	        *msg = NXLocalizedString("MauvaisTexte", "Impossible de dcoder le texte", Impossible de decoder le texte);
	}
	return self;
}

- window
{
	return window;
}

@end
